open Ldap_types
open Ldap_ooclient
open Tdir_types
open Ldap_error

let bind_result ?(code=`SUCCESS) ?(error="success") 
    ?(matched_dn="") ?(refs=None) msgid =
  {messageID=msgid;
   protocolOp=Bind_response
    {bind_result={result_code=code;
		  matched_dn="";
		  error_message=error;
		  ldap_referral=None};
     bind_serverSaslCredentials=None};
   controls=None}

let bind cfg {connections=connections;ad_schema=schema} conn_id message = 
  match message with
      {messageID=id;
       protocolOp=
	  Bind_request 
	    {bind_version=3;bind_name=dn;
	     bind_authentication=Simple cred}} ->	
	(try
	   let connection = Tdir_utils.connection cfg connections conn_id in
	     connection.con_adcon#bind ~cred:cred (Tdir_utils.translate_dn cfg dn);
	     connection.con_bound <- true;
	     bind_result id
	 with 
	     LDAP_Failure (c, e, {ext_matched_dn=mdn;ext_referral=refs}) ->
	       bind_result ~code:c ~error:e ~matched_dn:mdn ~refs:refs
		 message.messageID
	   | Tdir_utils.Util_failure msg -> 
	       bind_result ~code:`INVALID_DN_SYNTAX ~error:msg id
	   | LDAP_Decoder e -> 
	       bind_result ~code:`OTHER 
		 ~error:("AD Protocol error: " ^ e) message.messageID		 
	   | exn -> 
	       bind_result ~code:`OTHER 
		 ~error:("unhandled exception: " ^ (Printexc.to_string exn)) id)
    | {protocolOp=Bind_request {bind_version=2}} ->
	bind_result ~code:`UNWILLING_TO_PERFORM 
	  ~error:"Bind version not allowed" message.messageID
    | _ -> bind_result ~code:`PROTOCOL_ERROR ~error:"" message.messageID
