open Ldap_types
open Ldap_ooclient
open Tdir_types
open Str
open Syslog

let delete_res ?(referral=None) ?(code=`SUCCESS) ?(mdn="success") ?(err="") msg = 
  {messageID=msg.messageID;
   protocolOp=Delete_response {result_code=code;
			       matched_dn=mdn;
			       error_message=err;
			       ldap_referral=referral};
   controls=None}

let delete cfg {connections=connections;ad_schema=schema} conn_id message =
  match message with
      {messageID=msgid;
       protocolOp=Delete_request dn} ->
	if (List.exists (* make sure the dn meets the user's criteria *)
	      (fun rx -> Pcre.pmatch ~rex:rx dn) 
	      !cfg.cf_dn_must_not_match) || 
	   (not 
	      (List.exists
		 (fun rx -> Pcre.pmatch ~rex:rx dn)
		 !cfg.cf_dn_must_match)) ||
	   (not (Tdir_utils.match_object cfg dn))
	then begin
	  syslog !cfg.cf_syslog `LOG_DEBUG "dn does not meet criteria, not doing delete";
	  delete_res message
	end
	else begin
	  try
	    let connection = Tdir_utils.connection cfg connections conn_id in	   
	      connection.con_adcon#delete (Tdir_utils.translate_dn cfg dn);
	      delete_res message
	  with 
	      LDAP_Failure (code, err, {ext_matched_dn=mdn;ext_referral=refer}) ->
		delete_res ~code:code ~err:err ~mdn:mdn ~referral:refer message
	    | Tdir_utils.Util_failure msg -> 
		delete_res ~code:`INVALID_DN_SYNTAX ~err:msg message
	    | LDAP_Decoder e -> 
		delete_res ~code:`OTHER ~err:("AD Protocol error: " ^ e) message
	    | exn -> 
		delete_res message ~code:`OTHER 
		  ~err:("unhandled exception: " ^ (Printexc.to_string exn))
	end
    | _ -> delete_res ~code:`PROTOCOL_ERROR ~err:"expected delete request" message
