open Ldap_types
open Ldap_ooclient
open Tdir_types
open Str
open Syslog

let modrdn_res ?(referral=None) ?(code=`SUCCESS) ?(mdn="success") ?(err="") msg = 
  {messageID=msg.messageID;
   protocolOp=Modify_dn_response {result_code=code;
				  matched_dn=mdn;
				  error_message=err;
				  ldap_referral=referral};
   controls=None}

let modrdn cfg {connections=connections;ad_schema=schema} conn_id message = 
  match message with
      {messageID=msgid;
       protocolOp=
	  Modify_dn_request 
	    {modn_dn=current_dn;
	     modn_newrdn=new_rdn;
	     modn_deleteoldrdn=deleteoldrdn;
	     modn_newSuperior=new_superior}} ->
	if (List.exists (* make sure the dn meets the user's criteria *)
	      (fun rx -> Pcre.pmatch ~rex:rx current_dn) 
	      !cfg.cf_dn_must_not_match) || 
	   (not 
	      (List.exists
		 (fun rx -> Pcre.pmatch ~rex:rx current_dn)
		 !cfg.cf_dn_must_match)) ||
	   (not (Tdir_utils.match_object cfg current_dn))
	then begin
	  syslog !cfg.cf_syslog `LOG_DEBUG "dn does not meet criteria, not doing modrdn";
	  modrdn_res message
	end
	else begin
	  try
	    let connection = Tdir_utils.connection cfg connections conn_id in
	    let new_rdn = 
	      (replace_first 
		 (regexp_case_fold (!cfg.cf_imaginary_replica_rdn ^ " *= *"))
		 (!cfg.cf_real_replica_rdn ^ "=")
		 new_rdn)
	    in	   
	      connection.con_adcon#modrdn 
		~deleteoldrdn:true
		(Tdir_utils.translate_dn cfg current_dn)
		new_rdn;
	      modrdn_res message
	  with 
	      LDAP_Failure (code, err, {ext_matched_dn=mdn;ext_referral=refer}) ->
		modrdn_res ~code:code ~err:err ~mdn:mdn ~referral:refer message
	    | Tdir_utils.Util_failure msg -> 
		modrdn_res ~code:`INVALID_DN_SYNTAX ~err:msg message
	    | LDAP_Decoder e -> 
		modrdn_res ~code:`OTHER ~err:("AD Protocol error: " ^ e) message
	    | exn -> 
		modrdn_res ~code:`OTHER 
		  ~err:("unhandled exception: " ^ (Printexc.to_string exn)) message
	end
    | _ -> modrdn_res ~code:`PROTOCOL_ERROR ~err:"expected modrdn" message
