open Ldap_types
open Ldap_ooclient
open Ldap_funserver
open Ldap_schemaparser

(** raise when a utility fails for some reason *)
exception Util_failure of string

(** used to codefy a special case dn translation *)
type dn_translation_record = {
  dn_to_match: Pcre.regexp;
  dn_to_emit: string;
}

type attribute_mapping = {
  attribute_to_match: Pcre.regexp;
  attribute_to_emit: string;
}

type group_join_type = Uid_on_group 
		       | Gid_on_user 

type replication_type = Active_directory
			  | E_directory
			  | Standard_directory

(** configurable options *)
type config = {
  cf_read_con: ldapcon;
  cf_replica_type: replication_type;
  cf_port: int;
  cf_replica_hosts: string list;
  cf_replica_connect_timeout: int;
  cf_real_replica_base: string;
  cf_imaginary_replica_base: string;
  cf_real_replica_rdn: string;
  cf_imaginary_replica_rdn: string;
  cf_dn_special_cases: dn_translation_record list;
  cf_group_join_type: group_join_type;
  cf_imaginary_group_join_attr: string;
  cf_real_group_join_attr: string;
  cf_user_object_match: string; (** an objectclass, if present the object is a user *)
  cf_group_object_match: string; (** an objectclas, if present the object is a group *)
  cf_attributes_add_musts: (string * string list) list;
  cf_attributes_ignore_required: string list;
  cf_objectclasses_strip: string list;
  cf_attributes_strip: string list;
  cf_attributes_single_value: string list;
  cf_attribute_mappings: attribute_mapping list;
  cf_userpassword_filter: Pcre.regexp list; (** filter out matching userpassword values *)
  cf_dn_must_not_match: Pcre.regexp list; 
  (** ignore all changes, returning success, to any object who's dn
      matches one of these regexps *)
  cf_dn_must_match: Pcre.regexp list;
  cf_object_must_match: string option;
  cf_target_group_must_match: Pcre.regexp list;
  cf_target_group_must_not_match: Pcre.regexp list;
  cf_filter_directory: ldapcon option;
  cf_schemacheck_flavor: Ldap_ooclient.scflavor;
  cf_debugging: bool;
  cf_syslog: Syslog.t;
  cf_find_object_dn_cache_timeout: Int32.t;
}

(** maintains the state of one connection. These are stored in a hashtbl
    indexed by connection id in the server_context structure *)
type connection = {
  mutable con_bound: bool; (* have they bound *)
  mutable con_adcon: ldapcon; (* the corosponding connection to AD *)
}

(** info everyone needs to know *)
type server_context = {
  mutable ad_schema: schema;
  connections: (connection_id, connection) Hashtbl.t
}

type find_object_dn_cache_entry = {
  fdn_dn: string;
  fdn_time: Int32.t
}
